/* crt0.S

   initialization for exception environment and C runtime
   reentrant vector service routine for IRQ exceptions

   -----

   Copyright (c) Freescale Semiconductor, Inc., 2004

   This file is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to:

   Free Software Foundation, Inc.
   59 Temple Place, Suite 330
   Boston, MA  02111-1307  USA

*/

/*
$Id: $
*/

#define IRQ_DISABLE  0x80
#define FIQ_DISABLE  0x40

#define USR_MODE     0x10
#define FIQ_MODE     0x11
#define IRQ_MODE     0x12
#define SVC_MODE     0x13
#define ABT_MODE     0x17
#define UND_MODE     0x1B
#define SYS_MODE     0x1F

#define INTC_CLMASK  0x1E
#define INTC_SLMASK  0x1F
#define INTC_IRQIACK 0xEC

     .code   32
     .section ".text","ax"

     .global _reset
_reset:

     /* initialize exception environment */

     /* NOTE: These initializations assume that you have set up your stacks */
     /*       with an appropriate amount of memory for each operating mode. */
     /*       If you do not take the stack requirements for each mode into  */
     /*       account when performing this setup, you will encounter stack- */
     /*       related problems at runtime which are difficult to debug.     */

     mov       r0, #(IRQ_DISABLE|FIQ_DISABLE|FIQ_MODE)
     msr       cpsr, r0
     ldr       sp, .__FIQ_stack

     mov       r0, #(IRQ_DISABLE|FIQ_DISABLE|IRQ_MODE)
     msr       cpsr, r0
     ldr       sp, .__IRQ_stack

     mov       r0, #(IRQ_DISABLE|FIQ_DISABLE|ABT_MODE)
     msr       cpsr, r0
     ldr       sp, .__ABT_stack

     mov       r0, #(IRQ_DISABLE|FIQ_DISABLE|UND_MODE)
     msr       cpsr, r0
     ldr       sp, .__UND_stack

     mov       r0, #(IRQ_DISABLE|FIQ_DISABLE|SYS_MODE)
     msr       cpsr, r0
     ldr       sp, .__SYS_stack

     mov       r0, #(IRQ_DISABLE|FIQ_DISABLE|SVC_MODE)
     msr       cpsr, r0
     msr       spsr, r0       /* some LIBGCC1 funcs restore from SPSR */
     ldr       sp, .__SVC_stack

     .global  _start
_start:

     /* initialize C runtime environment */

     /* NOTE: These init routines assume that the .data and .bss segments   */
     /*       have been aligned on 4-byte boundaries.  See project.ld for   */
     /*       details.  If you do not align on a 4-byte boundary, this code */
     /*       will not work properly.                                       */

     /* copy data from ROM to RAM */
_init_data:
     ldr       r3, .__rom_data_start
     ldr       r4, .__ram_data_start
     ldr       r5, .__ram_data_end
     cmp       r4, r5                   /* bypass if no data to move        */
     beq       2f
1:   ldr       r0, [r3], #4             /* copy data four bytes at a time   */
     str       r0, [r4], #4
     cmp       r4, r5
     blt       1b
2:

     /* clear BSS */
_init_bss:
     ldr       r1, .__bss_start
     ldr       r2, .__bss_end
     mov       r0, #0
     cmp       r1, r2                   /* bypass if no .bss to clear       */
     beq       2f
1:   str       r0, [r1], #4             /* zero .bss four bytes at a time   */
     cmp       r1, r2
     blt       1b
2:
     bl        sysinit
     bl        main

     .global exit
     .global _exit
exit:
_exit:
     b       _exit

     .global vsr_IRQ
vsr_IRQ:
 
     /* enter in ARM state, IRQ mode                                        */
     /* pre-IRQ CPSR saved in SPSR_irq, PC+4 saved in LR_irq                */
 
     stmfd     sp!, {r0-r6, r12, lr}    /* LR_irq + 7 regs onto IRQ stack   */
     mrs       r6,  spsr                /* r6 := SPSR_irq                   */
 
     ldr       r4,  .__INTC_base        /* base address of INTC module      */
     ldrb      r5,  [r4, #INTC_SLMASK]  /* r5 := SLMASK                     */
     ldrb      r2,  [r4, #INTC_IRQIACK] /* r2 := IRQIACK                    */

     /* LR_irq, working and scratch registers are now saved on IRQ stack    */
     /* base addr of INTC, the SLMASK, and SPSR_irq are now in r4,r5,r6     */
     /* all other regs are non-volatile and will be preserved downstream    */

     /* switch to system mode before re-enabling IRQ exceptions             */

     msr       cpsr_c, #SYS_MODE        /* SYS mode, IRQ & FIQ enabled      */

     stmfd     sp!, {lr}                /* link register onto user stack    */
     subs      r2,  r2, #64             /* r2 := IRQ source number          */
     movpl     r3,  #0x40000000         /* address of jump table            */
     movpl     lr,  pc                  /* prepare link for branch          */
     ldrpl     pc,  [r3, r2, lsl #2]    /* jump to correct ISR              */
     ldmfd     sp!, {lr}                /* restore LR_usr from user stack   */

     /* switch back to IRQ mode, IRQ's disabled, to restore machine state   */
     msr       cpsr_c, #(IRQ_MODE|IRQ_DISABLE) 

     ldrb      r2,  [r4, #INTC_SLMASK]  /* r2 := SLMASK (the old CLMASK)    */
     strb      r5,  [r4, #INTC_SLMASK]  /* restore old SLMASK from r5       */
     strb      r2,  [r4, #INTC_CLMASK]  /* restore old CLMASK from r2       */
     msr       spsr,  r6                /* restore old SPSR_irq from r6     */
     ldmfd     sp!, {r0-r6, r12, lr}    /* restore registers from stack     */
     subs      pc,  lr,  #4             /* return from exception            */

/* create pointers to various objects */
.__FIQ_stack:       .word     __FIQ_stack
.__UND_stack:       .word     __UND_stack
.__ABT_stack:       .word     __ABT_stack
.__IRQ_stack:       .word     __IRQ_stack
.__SYS_stack:       .word     __SYS_stack
.__SVC_stack_base:  .word     __SVC_stack_base
.__SVC_stack:       .word     __SVC_stack

.__bss_start:       .word     __bss_start
.__bss_end:         .word     __bss_end
.__rom_data_start:  .word     __rom_data_start
.__ram_data_start:  .word     __ram_data_start
.__ram_data_end:    .word     __ram_data_end

.__INTC_base:       .word     0xFC048000

     .section ".stack"

/* a pointer to __stack_base is provided by the linker -- see "project.ld"  */

/* Set up stack space according to the requirements of this application.    */
/* Your application will be different, and will almost certainly require    */
/* a different stack configuration.                                         */

__stack_base:

     .balign 32

     .rept   16
     .long   0
     .endr
__FIQ_stack:

     .rept   16
     .long   0
     .endr
__UND_stack:

     .rept   16
     .long   0
     .endr
__ABT_stack:

     .rept   16
     .long   0
     .endr
__IRQ_stack:

     .rept   64
     .long   0
     .endr
__SYS_stack:

__SVC_stack_base:
     .rept   128
     .long   0
     .endr
__SVC_stack:

__STACK_END__:
