/* sysinit.c

   system-level hardware initialization
   includes init for PIT3 fast interrupt source

   -----

   Copyright (c) Freescale Semiconductor, Inc., 2004

   This file is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this file; if not, write to:

   Free Software Foundation, Inc.
   59 Temple Place, Suite 330
   Boston, MA  02111-1307  USA

*/

/*
$Id: $
*/

#include "project.h"

void
sysinit(void)
{

  int i, errflag;             /* loop index, retval from lib funcs */
  unsigned int esci_mode;     /* eSCI operating mode bit mask */

  /* ----- initialize system clock to FSYS (see project.h) */
  errflag = mac7101_crg_init ( FSYS );

  /* initialize PORT module to GPIO outputs for LEDs */
  for (i=12; i<=15; i++)
    PORTF_CONFIG(i) = GPIO_MODE | DDR_OUTPUT;

  PORTF_PORTDATA  = 0xF000;             /* turn off all LEDs */

  /* initialize eSCI module */
  esci_mode = ESCI_CR12_RE              /* enable receiver */
            | ESCI_CR12_TE;             /* enable transmitter */

  errflag = mac7101_sci_init (
    FSYS,                               /* system clock (in MHz) */
    ESCI_BASE,                          /* eSCI channel */
    115200,                             /* baud rate */
    esci_mode);                         /* mode mask (see above) */

  /* initialize PIT1 timer to produce interrupts */
  errflag = mac7101_pit_init (
    FSYS,                               /* system clock (in MHz) */
    1,                                  /* PIT channel (1..4) */
    (100*MSECONDS));                    /* period */

  /* initialize PIT2 timer to produce interrupts */
  errflag = mac7101_pit_init (
    FSYS,                               /* system clock (in MHz) */
    2,                                  /* PIT channel (1..4) */
    (10*USECONDS));                     /* period */

  /* initialize interrupt controller */
  /* configure levels 0-14 as IRQ, level 15 as FIQ, masking enabled */
  INTC_ICONFIG = INTC_CONFIG_EMASK
               | INTC_CONFIG_FIQDEF_15;

  INTC_ICR(PIT1_SRC) = 7;               /* make PIT1 a Level-7 int     */
  INTC_CIMR = PIT1_SRC;                 /* unmask PIT1 in CIMR         */

  INTC_ICR(PIT2_SRC) = 15;              /* make PIT2 a Level-15 int    */
  INTC_CIMR = PIT2_SRC;                 /* unmask PIT2 in CIMR         */

  /* set PIT3 to a higher priority than PIT1 and unmask it */
  /* (unmasking is not actually necessary because PIT3 is being forced) */
  INTC_ICR(PIT3_SRC) = 8;               /* make PIT3 a Level-8 int */
  INTC_CIMR = PIT3_SRC;                 /* unmask PIT3 in CIMR */

  /* enter the PIT1 ISR address in jump table */
  install_isr(LED_Blinker, PIT1_SRC);

  /* enter the PIT3 ISR address in jump table */
  install_isr(LED_Reverse, PIT3_SRC);

  /* read-modify-write the CPSR to enable IRQ exceptions */
  enable_irq();

  /* read-modify-write the CPSR to enable FIQ exceptions */
  enable_fiq();

  return;
}
