/* isr.c

   interrupt service routines for the sample project

   Enter via a vector service routine that uses the interrupt source 
   number to compute an offset into a jump table of function pointers.
   This number is available by reading the IRQIACK register.  See the
   file crt0.S for details.

   You can call other procedures from these ISRs, because the vector
   service routine has saved the scratch registers r0-r3 and ip (r12).
   (Make sure you enough stack to support any procedure calls.)

   -----

   Copyright (c) Freescale Semiconductor, Inc., 2004

   This file is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this file; if not, write to:

   Free Software Foundation, Inc.
   59 Temple Place, Suite 330
   Boston, MA  02111-1307  USA

*/

/*
$Id: $
*/

#include "project.h"

void
LED_Blinker (void)
{
  /* clear PIT1 flag by writing "1" */
  PIT_FLG = PIT_FLAG_TIF(1);

  /* set the current LED state
     LEDs are active LOW, so this will produce 3-on, 1-off */
  LEDstate = bit12_16 << LEDpos;

  /* reverse the LED state by forcing an PIT3 interrupt */
  /* this change will ONLY be visible if nested interrupts are supported */
  INTC_INTFRC |= INTC_IFRC(PIT3_SRC);

  /* write the LED state to the PORT pins */
  PORTF_PORTDATA = LEDstate;

  /* print the fixed text */
  SCI_putStr( ESCI_BASE, "\rLEDpos = " );

  /* print the LED position, left-justified in a 3-digit field */
  SCI_putStr( ESCI_BASE, u8_to_dec_lj( LEDpos ) );

  /* advance LED position */
  if (++LEDpos > 3)
    LEDpos = 0;

  /* return to vector service routine */
  return;
}

void
LED_Reverse (void)
{
  /* clear Interrupt Force Register flag bit */
  INTC_INTFRC &= ~(INTC_IFRC(PIT3_SRC));

  /* negate the LED state variable
     LEDs are active LOW, so this will produce 1-on, 3-off */
  LEDstate = ~(LEDstate);

  return;
}
