OUTPUT_FORMAT("elf32-bigarm")
OUTPUT_ARCH(arm)
ENTRY(_reset)

MEMORY {
  VECT (RX) : ORIGIN = 0x0000,     LENGTH = 0x400
  CFG  (RW) : ORIGIN = 0x0400,     LENGTH = 0x20
  TEXT (RX) : ORIGIN = 0x0420,     LENGTH = 512K - 0x420
  RAM  (RW) : ORIGIN = 0x40000000, LENGTH = 32K
}

SECTIONS
{

  .vectors : { *(.vectors) }     > VECT
  .config  : { *(.sgfm_config) } > CFG
  .text    : { *(.text) }        > TEXT
  .rodata  : { *(.rodata) }      > TEXT

  . = ALIGN(32 / 8);
  __rom_data_start = .;

  .data : AT(__rom_data_start)
  {
    __ram_data_start = .;

    /* IRQ handling requires the jump table to be at 0x40000000.
       If you don't put it there, then you must rewrite the vsr_IRQ 
       function in crt0.S so it can find the table. */

    jumptab.o(.data)

    *(.data .data.* .gnu.linkonce.d.*)

   /* crt0.S copies data 32 bits at a time, so alignment is critical.
      Align here to ensure that the .data section occupies space up to
      _end.  Align after .data to ensure correct alignment even if the
      .data section disappears because there are no input sections. */

   . = ALIGN(32 / 8);

  } > RAM

  . = ALIGN(32 / 8);
  __ram_data_end = .;
  __bss_start = .;

  .sbss :
  {
    __sbss_start = .;
    *(.dynsbss)
    *(.sbss .sbss.* .gnu.linkonce.sb.*)
    *(.scommon)
   . = ALIGN(32 / 8);
  } > RAM

  . = ALIGN(32 / 8);
  __sbss_end = .;

  .bss :
  {
   *(.dynbss)
   *(.bss .bss.* .gnu.linkonce.b.*)
   *(COMMON)
   . = ALIGN(32 / 8);
  } > RAM

  . = ALIGN(32 / 8);
  end = .;
  _end = .;
  __bss_end = . ;

  .stack :
  {
    _stack = .;
    *(.stack)
  } > RAM

}
