/*  serial.c

    board specific serial routines for the MAC7100 EVB

    -----

    Copyright (c) Freescale Semiconductor, Inc., 2004

    This file is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This file is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this file; if not, write to:

    Free Software Foundation, Inc.
    59 Temple Place, Suite 330
    Boston, MA  02111-1307  USA
*/

/*
$Id: $
*/

#include "serial.h"
#include "mac7111_esci.h"

/********************************************************************/
/* SCI_putStr  -  send a string to a specified serial port 

   takes base address of SCI module and pointer to char buffer
   returns number of characters sent
   calls SCI_putChar routine

   Example:
   SCI_putStr( ESCI_A_BASE, "Hello, World!\r\n" );

   Note: ensure the buffer is terminated with a null char
*/

unsigned int
SCI_putStr ( unsigned int esci_base, char* buffer )
{
    unsigned int count = 0;

    while (*buffer) {
        count++;
        SCI_putChar( esci_base, *(buffer++) );
    }

    return count;
}

/*****************************************************************/
char
SCI_getChar ( unsigned int esci_base )
{
    /* Wait until character has been received */
    while (!(ESCI_SR( esci_base ) & ESCI_SR_RDRF ))
	  ;

    /* clear the RDRF flag bit */
    ESCI_SR( esci_base ) = ESCI_SR_RDRF;

    /* return the character */
    return ESCI_DRL( esci_base );
}

/*****************************************************************/
void
SCI_putChar ( unsigned int esci_base, char ch )
{
    /* send the character */
    ESCI_DRL( esci_base ) = ch;

    /* wait until character is gone */
    while (!(ESCI_SR( esci_base ) & ESCI_SR_TDRE))
      ;

    /* clear the TDRE flag bit and return */
    ESCI_SR( esci_base ) = ESCI_SR_TDRE;
}
