/* mac7101_crg_init.c
   
   device driver to initialize the clock and reset generator

   -----

   Copyright (c) Freescale Semiconductor, Inc., 2004

   This file is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this file; if not, write to:

   Free Software Foundation, Inc.
   59 Temple Place, Suite 330
   Boston, MA  02111-1307  USA

*/

/*
$Id: $
*/

#include "typedefs.h"         // mac7101 register definitions
#include "mac7100_evb.h"      // mac7100 evb platform information
#include "mac7111_crg.h"      // mac7100 CRG module definitions
#include "mac7111_init.h"     // mac7100 LLD driver file header

int 
mac7101_crg_init(uint8_t f_sys)
{
  uint32_t i;

  /* this REFDV gives 1MHz resolution */
  CRG_REFDV = (uint8_t) (((FOSC * 2) - 1) & 0x0F);

  /* works for f_osc <= 8MHz */
  CRG_SYNR = (uint8_t) ((f_sys - 1) & 0x3F);

  /* wait a little while for PLL to lock, but not forever */
  for (i=0; i<0xFFFF; i++) {
    if ((CRG_CRGFLG & CRGFLG_LOCK) == CRGFLG_LOCK) {
      i = 0xFFFF;
    }
  };

  if ((CRG_CRGFLG & CRGFLG_LOCK) == CRGFLG_LOCK) {
    /* select the PLL as sysclock */
    CRG_CLKSEL |= CLKSEL_PLLSEL;
    return 0;
  } else {
    /* failed to achieve pll lock, so self-clock mode */
    return 1;
  }
}
