/* mac7101_pit_init.c
   
   device driver to initialize the periodic interval timer

   -----

   Copyright (c) Freescale Semiconductor, Inc., 2004

   This file is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this file; if not, write to:

   Free Software Foundation, Inc.
   59 Temple Place, Suite 330
   Boston, MA  02111-1307  USA

*/

/*
$Id: $
*/

#include "typedefs.h"
#include "mac7111_init.h"     // mac7100 LLD driver file header
#include "mac7111_pit.h"      // defines and macros for PIT module

int
mac7101_pit_init(
    uint8_t f_sys, 
    uint32_t chan, 
    uint32_t period)
{
  /* this driver only works for setting PIT interrupts on channel 1-4 */
  switch (chan)
  {
  case 1 ... 4:
    break;
  default:
    return 1;
  }

  /* clear MDIS bit to enable PIT */
  PIT_CTRL = 0;

  /* set the timer load value */
  PIT_TLVAL(chan) = ((period * f_sys / 2 ) - 1) & 0x00FFFFFF;

  /* do interrupts instead of DMA xfers */
  PIT_INTSEL |= PIT_INTSEL_ISEL(chan);

  /* enable the interrupt */
  PIT_INTEN |= PIT_INTEN_TIE(chan);

  /* enable the timer */
  PIT_EN |= PIT_EN_PEN(chan);

  return 0;
}
