/* mac7101_sci_init.c

   device driver to initialize an SCI module

   Inputs:    which_sci : start address of SCI module control registers
              f_sys     : system clock rate (in MHz)
              baudRate  : working baud rate of SCI.
                          can be 300, 600, 1200, 2400, 4800, 9600,
                          19200, 38400, 57600 or 115200
              opmode    : operating mode (bit mask for SCICR1, SCICR2)

   Outputs:   1         : there was an error in initializing the SCI
              0         : there were no errors

   -----

   Copyright (c) Freescale Semiconductor, Inc., 2004

   This file is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this file; if not, write to:

   Free Software Foundation, Inc.
   59 Temple Place, Suite 330
   Boston, MA  02111-1307  USA

*/

/*
$Id: $
*/

#include "typedefs.h"         /* standard integer type definitions */
#include "mac7111_init.h"     /* mac7100 LLD driver file header */
#include "mac7111_esci.h"     /* eSCI macros and definitions */
#include "mac7111_pim.h"      /* PIM macros and definitions */

int 
mac7101_sci_init (
    uint8_t  f_sys, 
    uint32_t esci_base, 
    uint32_t baud_rate,
    uint32_t opmode)
{

    uint16_t sbrd;            /* SCI baud rate divider */

    switch (baud_rate)
    {
    case 300:
    case 600:
    case 1200:
    case 2400:
    case 4800:
    case 9600:
    case 19200:
    case 38400:
    case 57600:
    case 115200:
        break;
    default:
        return 1;
    }

    switch (esci_base)
    {

    case ESCI_A_BASE:
        PORTG_CONFIG(2) = 0x0080;
        PORTG_CONFIG(3) = 0x0080;
        break;

    case ESCI_B_BASE:
        PORTG_CONFIG(0) = 0x0080;
        PORTG_CONFIG(1) = 0x0080;
        break;

    case ESCI_C_BASE:
        PORTG_CONFIG(14) = 0x0080;
        PORTG_CONFIG(15) = 0x0080;
        break;

    case ESCI_D_BASE:
        PORTG_CONFIG(12) = 0x0080;
        PORTG_CONFIG(13) = 0x0080;
        break;

    default:

        return 1;
    }
    
    /* Calculate the SCI baud rate divider */
    sbrd = (uint16_t)((f_sys * 1000000)/(2 * baud_rate * 16));

    /* Maximum baud rate divider is 8191 */
    if (sbrd > 8191)
        return 1;

    /* initialize SCI module */
    ESCI_CR3(esci_base) = 0;            /* clear MDIS bit to enable SCI */
    ESCI_LINCTRL(esci_base) = 0;        /* disable LIN mode */
    ESCI_BD(esci_base) = sbrd;          /* set baud rate */
    ESCI_CR12(esci_base) = opmode;      /* set user-defined operation mode */

    return 0;
}
