/*
$Id: mac7111_intc.h,v 1.1 2004/04/26 16:45:56 ra4951 Exp $
*/

#ifndef _MAC7111_INTC_H_
#define _MAC7111_INTC_H_

/* Interrupt Controller Definitions */
#define INTC_BASE      (0xFC048000)

#define IPRH_OFFSET    (0x0000)
#define IPRL_OFFSET    (0x0004)
#define IMRH_OFFSET    (0x0008)
#define IMRL_OFFSET    (0x000C)
#define INTFRCH_OFFSET (0x0010)
#define INTFRCL_OFFSET (0x0014)
#define ICONFIG_OFFSET (0x001B)
#define SIMR_OFFSET    (0x001C)
#define CIMR_OFFSET    (0x001D)
#define ICR_OFFSET     (0x0040)
#define IRQIACK_OFFSET (0x00EC)
#define FIQIACK_OFFSET (0x00F0)

#define INTC_IPRH      (*(volatile unsigned int *) (INTC_BASE + IPRH_OFFSET))
#define INTC_IPRL      (*(volatile unsigned int *) (INTC_BASE + IPRL_OFFSET))
#define INTC_IMRH      (*(volatile unsigned int *) (INTC_BASE + IMRH_OFFSET))
#define INTC_IMRL      (*(volatile unsigned int *) (INTC_BASE + IMRL_OFFSET))
#define INTC_IFRC(x)   (1ULL << (x & 63))
#define INTC_INTFRC    (*(volatile unsigned long long *) (INTC_BASE + INTFRCH_OFFSET))
#define INTC_INTFRCH   (*(volatile unsigned int *) (INTC_BASE + INTFRCH_OFFSET))
#define INTC_INTFRCL   (*(volatile unsigned int *) (INTC_BASE + INTFRCL_OFFSET))
#define INTC_ICONFIG   (*(volatile unsigned char *) (INTC_BASE + ICONFIG_OFFSET))
#define INTC_IRQIACK   (*(volatile unsigned char *) (INTC_BASE + IRQIACK_OFFSET))
#define INTC_FIQIACK   (*(volatile unsigned char *) (INTC_BASE + FIQIACK_OFFSET))
#define INTC_ICR(src)  (*(volatile unsigned char *) (INTC_BASE + ICR_OFFSET + src))
#define INTC_SIMR      (*(volatile unsigned char *) (INTC_BASE + SIMR_OFFSET))
#define INTC_CIMR      (*(volatile unsigned char *) (INTC_BASE + CIMR_OFFSET))

#define INTC_CONFIG_EMASK     (0x20)
#define INTC_CONFIG_FIQDEF_15 (0x0F)
#define INTC_CONFIG           (0x0018)

/* hardware interrupt source vector numbers */
#define eDMA0_SRC        (0)
#define eDMA1_SRC        (1)
#define eDMA2_SRC        (2)
#define eDMA3_SRC        (3)
#define eDMA4_SRC        (4)
#define eDMA5_SRC        (5)
#define eDMA6_SRC        (6)
#define eDMA7_SRC        (7)
#define eDMA8_SRC        (8)
#define eDMA9_SRC        (9)
#define eDMA10_SRC       (10)
#define eDMA11_SRC       (11)
#define eDMA12_SRC       (12)
#define eDMA13_SRC       (13)
#define eDMA14_SRC       (14)
#define eDMA15_SRC       (15)
#define eDMA_Error_SRC   (16)
#define MCM_SWT_SRC      (17)
#define CRG_SRC          (18)
#define PIT1_SRC         (19)
#define PIT2_SRC         (20)
#define PIT3_SRC         (21)
#define PIT4_RTI_SRC     (22)
#define VREG_SRC         (23)
#define CAN_A_MB_SRC     (24)
#define CAN_A_MB14_SRC   (25)
#define CAN_A_Error_SRC  (26)
#define CAN_B_MB_SRC     (27)
#define CAN_B_MB14_SRC   (28)
#define CAN_B_Error_SRC  (29)
#define CAN_C_MB_SRC     (30)
#define CAN_C_MB14_SRC   (31)
#define CAN_C_Error_SRC  (32)
#define CAN_D_MB_SRC     (33)
#define CAN_D_MB14_SRC   (34)
#define CAN_D_Error_SRC  (35)
#define I2C_SRC          (36)
#define DSPI_A_SRC       (37)
#define DSPI_B_SRC       (38)
#define eSCI_A_SRC       (39)
#define eSCI_B_SRC       (40)
#define eSCI_C_SRC       (41)
#define eSCI_D_SRC       (42)
#define eMIOS0_SRC       (43)
#define eMIOS1_SRC       (44)
#define eMIOS2_SRC       (45)
#define eMIOS3_SRC       (46)
#define eMIOS4_SRC       (47)
#define eMIOS5_SRC       (48)
#define eMIOS6_SRC       (49)
#define eMIOS7_SRC       (50)
#define eMIOS8_SRC       (51)
#define eMIOS9_SRC       (52)
#define eMIOS10_SRC      (53)
#define eMIOS11_SRC      (54)
#define eMIOS12_SRC      (55)
#define eMIOS13_SRC      (56)
#define eMIOS14_SRC      (57)
#define eMIOS15_SRC      (58)
#define ATD_SRC          (59)
#define CFM_SRC          (60)
#define PIM_SRC          (61)
#define IRQ_SRC          (62)
#define XIRQ_SRC         (63)

/* install_isr: install an interrupt service routine in the jump table
   
   Usage:   install_isr(<name of ISR>, <interrupt source number>);
   Example: Install an ISR named "LED_Blinker" for the PIT1 timer:
       install_isr(LED_Blinker, PIT1_SRC);
*/

#define RAM_BASE (0x40000000)
#define JUMPTAB RAM_BASE
#define install_isr(isr,source) (*(void **)(JUMPTAB + (4*source)) = (isr))

/* these macros only work with the GCC compiler */
#ifdef __GNUC__

#define push_spsr_irq(__arg)                    \
    asm volatile (                              \
        "mrs %0,spsr;"                          \
        : "=r"(__arg)                           \
        );

#define pop_spsr_irq(__arg)                     \
    asm volatile (                              \
        "msr spsr,%0;"                          \
        :                                       \
        : "r"(__arg)                            \
        );

#define disable_interrupts(_old_)               \
    asm volatile (                              \
        "mrs %0,cpsr;"                          \
        "mrs r4,cpsr;"                          \
        "orr r4,r4,#0xC0;"                      \
        "msr cpsr,r4"                           \
        : "=r"(_old_)                           \
        :                                       \
        : "r4"                                  \
        );
#define disable_irq()                           \
    asm volatile (                              \
        "mrs r3,cpsr;"                          \
        "orr r3,r3,#0x80;"                      \
        "msr cpsr,r3"                           \
        :                                       \
        :                                       \
        : "r3"                                  \
        );
#define enable_irq()                            \
    asm volatile (                              \
        "mrs r3,cpsr;"                          \
        "bic r3,r3,#0x80;"                      \
        "msr cpsr,r3"                           \
        :                                       \
        :                                       \
        : "r3"                                  \
        );
#define disable_fiq()                           \
    asm volatile (                              \
        "mrs r3,cpsr;"                          \
        "orr r3,r3,#0x40;"                      \
        "msr cpsr,r3"                           \
        :                                       \
        :                                       \
        : "r3"                                  \
        );
#define enable_fiq()                            \
    asm volatile (                              \
        "mrs r3,cpsr;"                          \
        "bic r3,r3,#0x40;"                      \
        "msr cpsr,r3"                           \
        :                                       \
        :                                       \
        : "r3"                                  \
        );
#define enable_interrupts()                     \
    asm volatile (                              \
        "mrs r3,cpsr;"                          \
        "bic r3,r3,#0xC0;"                      \
        "msr cpsr,r3"                           \
        :                                       \
        :                                       \
        : "r3"                                  \
        );
#define restore_interrupts(_old_)               \
    asm volatile (                              \
        "mrs r3,cpsr;"                          \
        "and r4,%0,#0xC0;"                      \
        "bic r3,r3,#0xC0;"                      \
        "orr r3,r3,r4;"                         \
        "msr cpsr,r3"                           \
        :                                       \
        : "r"(_old_)                            \
        : "r3", "r4"                            \
        );
#define query_interrupts(_old_)                 \
    asm volatile (                              \
        "mrs r4,cpsr;"                          \
        "and r4,r4,#0xC0;"                      \
        "eor %0,r4,#0xC0;"                      \
        : "=r"(_old_)                           \
        :                                       \
        : "r4"                                  \
        );
#endif /* __GNUC__ */
#endif /* _MAC7111_INTC_H_ */

